@@ -7,7 +7,8 @@
 #'
 #' @usage get.oc.comb(target, p.true, ncohort, cohortsize, n.earlystop=NULL, startdose=c(1, 1),
 #'                    titration=FALSE,p.saf=0.6*target, p.tox=1.4*target, cutoff.eli=0.95,
-#'                    extrasafe=FALSE,offset=0.05, ntrial=1000, mtd.contour=FALSE, seed=6)
+#'                    extrasafe=FALSE,offset=0.05, ntrial=1000, mtd.contour=FALSE,
+#'                    boundMTD=FALSE, seed=6)
 #'
 #' @param target the target DLT rate
 #' @param p.true a \code{J*K} matrix \code{(J<=K)} containing the true toxicity probabilities of
@@ -36,6 +37,8 @@
 #' @param ntrial the total number of trials to be simulated
 #' @param mtd.contour set \code{mtd.contour=TRUE} to select the MTD contour (claiming multiple MTDs).
 #'                    Otherwise, BOIN design is used to search for a single MTD.
+#' @param boundMTD set \code{boundMTD=TRUE} to impose the condition: the isotonic estimate of toxicity
+#'                 probability for the selected MTD must be less than de-escalation boundary.
 #' @param seed the random seed for simulation
 #'
 #' @details The operating characteristics of the BOIN design or waterfall design are generated by
@@ -76,6 +79,7 @@
 #' (6) the total number of toxicities observed for the trial (\code{$totaltox})
 #' (7) the pecentage of correct selection (\code{$pcs}),
 #' (8) the total percentage of patients treated at the MTD (\code{$npercent}).
+#' (9) the percentage of early stopping without selecting the MTD (\code{$percentstop})
 #' For the the waterfall design, including:
 #' (1) true toxicity probability at each dose level (\code{$p.true}),
 #' (2) selection percentage of dose combinations (\code{$selpercent}),
@@ -98,7 +102,7 @@
 #'       \code{get.oc()} are strongly recommended, and generally yield excellent operating characteristics.
 #'
 #'
-#' @author Suyu Liu, Liangcai Zhang and Ying Yuan
+#' @author Suyu Liu, Liangcai Zhang, Yanhong Zhou, and Ying Yuan
 #'
 #' @references Liu S. and Yuan, Y. (2015). Bayesian Optimal Interval Designs for Phase I Clinical
 #'             Trials, \emph{Journal of the Royal Statistical Society: Series C}, 64, 507-523.
@@ -138,7 +142,7 @@
 #'
 #' ## get the operating characteristics with titration for BOIN design
 #' oc.comb <- get.oc.comb(target=0.3, p.true, ncohort=20, cohortsize=3,
-#'    n.earlystop=12, startdose=c(1,1), ntrial=100)
+#'    n.earlystop=12, startdose=c(1,1), titration=TRUE, ntrial=100)
 #' summary(oc.comb)
 #' plot(oc.comb)
 #'
@@ -156,12 +160,12 @@
 get.oc.comb <- function (target, p.true, ncohort, cohortsize, n.earlystop = NULL,
                          startdose = c(1, 1), titration = FALSE, p.saf = 0.6 * target,
                          p.tox = 1.4 * target, cutoff.eli = 0.95, extrasafe = FALSE,
-                         offset = 0.05, ntrial = 1000, mtd.contour = FALSE, seed = 6)
+                         offset = 0.05, ntrial = 1000, mtd.contour = FALSE, boundMTD=FALSE,seed = 6)
 {
   get.oc.comb.boin <- function(target, p.true, ncohort, cohortsize,
                                n.earlystop = 100, startdose = c(1, 1), titration = FALSE,
                                p.saf = 0.6 * target, p.tox = 1.4 * target, cutoff.eli = 0.95,
-                               extrasafe = FALSE, offset = 0.05, ntrial = 1000) {
+                               extrasafe = FALSE, offset = 0.05, boundMTD=FALSE,ntrial = 1000) {
     JJ = nrow(p.true)
     KK = ncol(p.true)
     if (JJ > KK)
@@ -373,7 +377,8 @@ get.oc.comb <- function (target, p.true, ncohort, cohortsize, n.earlystop = NULL
         dselect[trial, ] = c(99, 99)
       }else {
         selcomb = select.mtd.comb.boin(target, n, y,
-                                       cutoff.eli, extrasafe, offset, mtd.contour = FALSE)$MTD
+                                       cutoff.eli, extrasafe, offset,
+                                       boundMTD=boundMTD,p.tox=p.tox,mtd.contour = FALSE)$MTD
         dselect[trial, 1] = selcomb[1]
         dselect[trial, 2] = selcomb[2]
       }
@@ -400,7 +405,8 @@ get.oc.comb <- function (target, p.true, ncohort, cohortsize, n.earlystop = NULL
                  npatients = round(apply(N, c(1, 2), mean),2), ntox = round(apply(Y, c(1, 2), mean), 2),
                  totaltox = round(sum(Y)/ntrial, 1), totaln = round(sum(N)/ntrial,1),
                  pcs = paste(round(sum(selpercent[which(abs(p.true -target) == min(abs(p.true - target)), arr.ind = TRUE)]),1), "%", sep = ""),
-                 npercent = paste(round(sum(nptsdose[which(abs(p.true -target) == min(abs(p.true - target)), arr.ind = TRUE)])/sum(nptsdose) *100, 1), "%", sep = ""), flowchart = TRUE)
+                 npercent = paste(round(sum(nptsdose[which(abs(p.true -target) == min(abs(p.true - target)), arr.ind = TRUE)])/sum(nptsdose) *100, 1), "%", sep = ""),
+                 percentstop=100-sum(round(selpercent,2)),flowchart = FALSE)
       rownames(out$npatients) = paste("DoseA", 1:dim(p.true)[1],
                                       sep = "")
       colnames(out$npatients) = paste("DoseB", 1:dim(p.true)[2],
@@ -432,13 +438,18 @@ get.oc.comb <- function (target, p.true, ncohort, cohortsize, n.earlystop = NULL
       out = list(p.true = round(t(p.true), 2), selpercent = round(t(selpercent),2),
                  npatients = round(t(apply(N, c(1, 2), mean)), 2), ntox = round(t(apply(Y, c(1, 2), mean)),2),
                  totaltox = round(sum(Y)/ntrial, 1), totaln = round(sum(N)/ntrial,1), pcs = paste(round(sum(selpercent[which(abs(p.true -target) == min(abs(p.true - target)), arr.ind = TRUE)]),
-                 1), "%"), npercent = paste(round(sum(nptsdose[which(abs(p.true -target) == min(abs(p.true - target)), arr.ind = TRUE)])/sum(nptsdose) *100, 1), "%"), flowchart = TRUE)
+                 1), "%"), npercent = paste(round(sum(nptsdose[which(abs(p.true -target) == min(abs(p.true - target)), arr.ind = TRUE)])/sum(nptsdose) *100, 1), "%"),
+                 percentstop=100-sum(round(selpercent,2)),
+                 flowchart = FALSE)
 
       return(out)
     }
   }
   select.mtd.comb.boin <- function(target, npts, ntox, cutoff.eli = 0.95,
-                                   extrasafe = FALSE, offset = 0.05, mtd.contour = FALSE) {
+                                   extrasafe = FALSE, offset = 0.05,
+                                   boundMTD=FALSE,p.tox=1.4*target,
+                                   mtd.contour = FALSE) {
+    lambda_d = log((1 - target)/(1 - p.tox))/log(p.tox * (1 -target)/(target * (1 - p.tox)))
     y = ntox
     n = npts
     if (nrow(n) > ncol(n) | nrow(y) > ncol(y)) {
@@ -465,11 +476,15 @@ get.oc.comb <- function (target, p.true, ncohort, cohortsize, n.earlystop = NULL
         }
       }
     }
+
+    selectdose=NULL
     if (elimi[1] == 1) {
+
       selectdose = c(99, 99)
       selectdoses = matrix(selectdose, nrow = 1)
-    }
-    else {
+
+    }else {
+
       phat = (y + 0.05)/(n + 0.1)
       phat = Iso::biviso(phat, n + 0.1, warn = TRUE)[,
                                                      ]
@@ -480,6 +495,20 @@ get.oc.comb <- function (target, p.true, ncohort, cohortsize, n.earlystop = NULL
                                                      each = dim(n)[2], len = length(n)), dim(n)[1],
                                                  byrow = T) + matrix(rep(1:dim(n)[2], each = dim(n)[1],
                                                                          len = length(n)), dim(n)[1]))
+
+
+      if(boundMTD){
+
+        if(all(phat[n!=0]>=lambda_d)){
+          selectdose = c(99, 99)
+
+
+          selectdoses = matrix(selectdose, nrow = 1)
+        }else{
+          phat[phat>=lambda_d]=10}}
+
+      if(is.null(selectdose)){
+
       phat[n == 0] = 10
       selectdose = which(abs(phat - target) == min(abs(phat -
                                                          target)), arr.ind = TRUE)
@@ -525,6 +554,8 @@ get.oc.comb <- function (target, p.true, ncohort, cohortsize, n.earlystop = NULL
       }
       selectdoses = matrix(selectdoses[selectdoses[, 2] !=
                                          99, ], ncol = 2)
+      }
+
       colnames(selectdoses) = c("DoseA", "DoseB")
     }
     if (mtd.contour == FALSE) {
@@ -560,8 +591,11 @@ get.oc.comb <- function (target, p.true, ncohort, cohortsize, n.earlystop = NULL
       }
     }
   }
-  waterfall.subtrial.mtd <- function(target, npts, ntox, cutoff.eli = 0.95,
-                                     extrasafe = FALSE, offset = 0.05, temp) {
+  waterfall.subtrial.mtd <- function(target,npts, ntox, cutoff.eli = 0.95,
+                                     extrasafe = FALSE, offset = 0.05, boundMTD=FALSE, p.tox, temp) {
+
+    lambda_d = log((1 - target)/(1 - p.tox))/log(p.tox * (1 -target)/(target * (1 - p.tox)))
+
     b.e = temp[2, ]
     pava <- function(x, wt = rep(1, length(x))) {
       n <- length(x)
@@ -619,19 +653,34 @@ get.oc.comb <- function (target, p.true, ncohort, cohortsize, n.earlystop = NULL
                                                              0.1)^2 * (n.adm + 0.1 + 1))
       phat = pava(phat, wt = 1/phat.var)
       phat = phat + (1:length(phat)) * 1e-10
-      selectd = sort(abs(phat - target), index.return = T)$ix[1]
-      selectdose = adm.index[selectd]
-      if (y[selectdose] <= b.e[n[selectdose]]) {
-        is.escalation = 1
+      if(boundMTD){
+        if(all(phat>=lambda_d)){selectdose=99}else{
+          phat=phat[phat<lambda_d]
+          selectd = sort(abs(phat - target), index.return = T)$ix[1]
+          selectdose = adm.index[selectd]
+        }
+
+      }else{
+        selectd = sort(abs(phat - target), index.return = T)$ix[1]
+        selectdose = adm.index[selectd]
       }
+      # selectd = sort(abs(phat - target), index.return = T)$ix[1]
+      # selectdose = adm.index[selectd]
+      if(selectdose!=99){
+        if (y[selectdose] <= b.e[n[selectdose]]) {
+          is.escalation = 1
+        }
+      }
+
     }
     list(selectdose = selectdose, is.escalation = is.escalation)
   }
+
   waterfall.subtrial <- function(target, p.true, dosespace,
                                  npts, ntox, elimi, ncohort, cohortsize, n.earlystop = 20,
                                  startdose = 1, p.saf = 0.6 * target, p.tox = 1.4 * target,
                                  cutoff.eli = 0.95, extrasafe = FALSE, offset = 0.05,
-                                 totaln, titration.first.trial = FALSE, temp) {
+                                 totaln, titration.first.trial = FALSE, temp,boundMTD=FALSE) {
     ndoses1 = nrow(p.true)
     ndoses2 = ncol(p.true)
     p.truee = p.true[dosespace]
@@ -732,8 +781,9 @@ get.oc.comb <- function (target, p.true, ncohort, cohortsize, n.earlystop = NULL
       elm = rep(1, ndose)
     }
     else {
+
       wsmtd = waterfall.subtrial.mtd(target, n, y, cutoff.eli,
-                                     extrasafe, offset, temp)
+                                     extrasafe, offset, boundMTD=boundMTD, p.tox,temp)
       selectdose = wsmtd$selectdose
       is.escalation = wsmtd$is.escalation
     }
@@ -749,7 +799,7 @@ get.oc.comb <- function (target, p.true, ncohort, cohortsize, n.earlystop = NULL
   get.oc.comb.waterfall <- function(p.true, target, ncohort,
                                     cohortsize, n.earlystop = 12, cutoff.eli = 0.95, p.saf = 0.6 *
                                       target, p.tox = 1.4 * target, titration = FALSE,
-                                    extrasafe = FALSE, offset = 0.05, ntrial = 1000) {
+                                    extrasafe = FALSE, offset = 0.05, boundMTD=FALSE,ntrial = 1000) {
     temp = get.boundary(target, ncohort = 150, cohortsize = 1,
                         cutoff.eli = cutoff.eli, extrasafe = extrasafe)$boundary_tab
     JJ = nrow(p.true)
@@ -854,7 +904,7 @@ get.oc.comb <- function (target, p.true, ncohort, cohortsize, n.earlystop = NULL
                                       startdose = startdose, p.saf = p.saf, p.tox = p.tox,
                                       cutoff.eli = cutoff.eli, extrasafe = extrasafe,
                                       offset = offset, totaln = totaln, titration.first.trial = titration.first.trial,
-                                      temp = temp)
+                                      temp = temp,boundMTD=boundMTD)
         selectdose = ifelse(subtrial$selectdose == 99,
                             99, dosespace[subtrial$selectdose])
         if (selectdose == 99)
@@ -879,7 +929,7 @@ get.oc.comb <- function (target, p.true, ncohort, cohortsize, n.earlystop = NULL
                                            cohortsize = cohortsize, n.earlystop = n.earlystop,
                                            startdose = startdose, p.saf = p.saf, p.tox = p.tox,
                                            cutoff.eli, extrasafe, offset, totaln = totaln,
-                                           temp = temp)
+                                           temp = temp,boundMTD=boundMTD)
             selectdose1 = ifelse(subtrial1$selectdose ==
                                    99, selectdose, dosespace1[subtrial1$selectdose])
             if (selectdose1 == 99)
@@ -1042,7 +1092,7 @@ get.oc.comb <- function (target, p.true, ncohort, cohortsize, n.earlystop = NULL
                                                                                                   1), "%", sep = ""), npercent.below.contour = paste(round(100 *
                                                                                                                                                              sum(nptsdose[less.than.contour])/sum(nptsdose),
                                                                                                                                                            1), "%", sep = ""), pcs.contour = paste(round(100 *
-                                                                                                                                                                                                           nselpercent/ntrial, 1), "%", sep = ""), flowchart = TRUE)
+                                                                                                                                                                                                           nselpercent/ntrial, 1), "%", sep = ""), flowchart = FALSE)
       }
       outnew = c(out, out2)
       class(outnew)<-"boin"
@@ -1092,7 +1142,7 @@ get.oc.comb <- function (target, p.true, ncohort, cohortsize, n.earlystop = NULL
                                       format = "f", width = 5), c(1, 2), as.numeric),
                  totaltox = as.numeric(formatC(sum(ntoxdose/ntrial),
                                                digits = 1, format = "f")), totaln = as.numeric(formatC(sum(nptsdose/ntrial),
-                                                                                                       digits = 1, format = "f")), flowchart = TRUE)
+                                                                                                       digits = 1, format = "f")), flowchart = FALSE)
       outnew = c(out, out2)
       class(outnew)<-"boin"
       return(outnew)
@@ -1124,7 +1174,7 @@ get.oc.comb <- function (target, p.true, ncohort, cohortsize, n.earlystop = NULL
                             ncohort = sum(ncohort), cohortsize = cohortsize,
                             n.earlystop = n.earlystop, startdose = startdose,
                             titration = titration, p.saf = p.saf, p.tox = p.tox,
-                            cutoff.eli = cutoff.eli, extrasafe = extrasafe, offset = offset,
+                            cutoff.eli = cutoff.eli, extrasafe = extrasafe, offset = offset,boundMTD=boundMTD,
                             ntrial = ntrial)
     class(out)<-"boin"
     return(out)
@@ -1152,7 +1202,7 @@ get.oc.comb <- function (target, p.true, ncohort, cohortsize, n.earlystop = NULL
     out=get.oc.comb.waterfall(p.true = p.true, target = target,
                           ncohort = ncohort, cohortsize = cohortsize, n.earlystop = n.earlystop,
                           cutoff.eli = cutoff.eli, p.saf = p.saf, p.tox = p.tox,
-                          titration = titration, extrasafe = extrasafe, offset = offset,
+                          titration = titration, extrasafe = extrasafe, offset = offset,boundMTD=boundMTD,
                           ntrial = ntrial)
     class(out)<-"boin"
     return(out)
